@@ -22,13 +22,6 @@ extern int EnvC;
 extern int argc;
 extern char **argv;
 
-void
-macos_error_cleanup( void )
-{
-	Lock_Cleanup();
-	RemoveConsole();		// FIXME - Ugly, but necessary until MW fixes _exit
-}
-
 void
 InitializeMacToolbox( void )
 {
@@ -59,8 +52,6 @@ MacOS_Initialize( int *argc, char ***argv )
 	#else
 	*argc = ccommand(argv);
 	#endif
-	
-	error_set_cleanup (macos_error_cleanup);
 }
 
 void
