@@ -33,6 +33,7 @@
  * edited by the user, if necessary (when the repository is moved, e.g.)
  */
 
+#include <assert.h>
 #include "cvs.h"
 
 static char *findslash PROTO((char *start, char *p));
@@ -241,6 +242,13 @@ checkout (argc, argv)
 	    error (1, 0, "tag `%s' must be a symbolic tag", tag);
     }
 
+#ifdef SERVER_SUPPORT
+    if (server_active && where != NULL)
+    {
+	server_pathname_check (where);
+    }
+#endif
+
     if (!safe_location()) {
         error(1, 0, "Cannot check out files into the repository itself");
     }
@@ -427,6 +435,10 @@ struct dir_to_build
     /* The path to the directory.  */
     char *dirpath;
 
+    /* If set, don't build the directory, just change to it.
+       The caller will also want to set REPOSITORY to NULL.  */
+    int just_chdir;
+
     struct dir_to_build *next;
 };
 
@@ -580,7 +592,11 @@ checkout_proc (pargc, argv, where_orig, mwhere, mfile, shorten,
 	    (void) strcat (where, "/");
 	}
 
-	if (mwhere != NULL)
+	/* If the -d flag in the modules file specified an absolute
+           directory, let the user override it with the command-line
+           -d option. */
+
+	if ((mwhere != NULL) && (! isabsolute (mwhere)))
 	    (void) strcat (where, mwhere);
 	else
 	    (void) strcat (where, argv[0]);
@@ -718,23 +734,63 @@ internal error: %s doesn't start with %s in checkout_proc",
 	head->repository = NULL;
 	head->dirpath = xstrdup (where);
 	head->next = NULL;
+	head->just_chdir = 0;
 
 
 	/* Make a copy of the repository name to play with. */
 	reposcopy = xstrdup (repository);
 
-	/* FIXME: this should be written in terms of last_component instead
-	   of hardcoding '/'.  This presumably affects OS/2, NT, &c, if
-	   the user specifies '\'.  Likewise for the call to findslash.  */
-	cp = strrchr (where, '/');
-	while (cp != NULL)
+	/* FIXME: this should be written in terms of last_component
+	   instead of hardcoding '/'.  This presumably affects OS/2,
+	   NT, &c, if the user specifies '\'.  Likewise for the call
+	   to findslash.  */
+	cp = where + strlen (where);
+	while (1)
 	{
 	    struct dir_to_build *new;
+
+	    cp = findslash (where, cp - 1);
+	    if (cp == NULL)
+		break;		/* we're done */
+
 	    new = (struct dir_to_build *)
 		xmalloc (sizeof (struct dir_to_build));
 	    new->dirpath = xmalloc (strlen (where));
-	    strncpy (new->dirpath, where, cp - where);
-	    new->dirpath[cp - where] = '\0';
+
+	    /* If the user specified an absolute path for where, the
+               last path element we create should be the top-level
+               directory. */
+
+	    if (cp - where)
+	    {
+		strncpy (new->dirpath, where, cp - where);
+		new->dirpath[cp - where] = '\0';
+	    }
+	    else
+	    {
+		/* where should always be at least one character long. */
+		assert (strlen (where));
+		strcpy (new->dirpath, "/");
+	    }
+	    new->next = head;
+	    head = new;
+
+	    /* If where consists of multiple pathname components,
+	       then we want to just cd into it, without creating
+	       directories or modifying CVS directories as we go.
+	       In CVS 1.9 and earlier, the code actually does a
+	       CVS_CHDIR up-front; I'm not going to try to go back
+	       to that exact code but this is somewhat similar
+	       in spirit.  */
+	    if (where_orig != NULL
+		&& cp - where < strlen (where_orig))
+	    {
+		new->repository = NULL;
+		new->just_chdir = 1;
+		continue;
+	    }
+
+	    new->just_chdir = 0;
 
 	    /* Now figure out what repository directory to generate.
                The most complete case would be something like this:
@@ -746,12 +802,13 @@ internal error: %s doesn't start with %s in checkout_proc",
 	         cvs co -d what/ever -N foo
 	       
 	       The results in the CVS/Repository files should be:
-	         .     -> .          (this is where we executed the cmd)
-		 what  -> Emptydir   (generated dir -- not in repos)
+	         .     -> (don't touch CVS/Repository)
+			  (I think this case might be buggy currently)
+		 what  -> (don't touch CVS/Repository)
 		 ever  -> .          (same as "cd what/ever; cvs co -N foo")
 		 bar   -> Emptydir   (generated dir -- not in repos)
 		 baz   -> quux       (finally!) */
-	    
+
 	    if (strcmp (reposcopy, CVSroot_directory) == 0)
 	    {
 		/* We can't walk up past CVSROOT.  Instead, the
@@ -810,34 +867,57 @@ internal error: %s doesn't start with %s in checkout_proc",
 		    }
 		}
 	    }
-	    
-	    new->next = head;
-	    head = new;
-
-	    cp = findslash (where, cp - 1);
 	}
 
 	/* clean up */
 	free (reposcopy);
 
-	/* The top-level CVSADM directory should always be
-           CVSroot_directory.  Create it.
-	
-	   It may be argued that we shouldn't set any sticky bits for
-	   the top-level repository.  FIXME?  */
-	build_one_dir (CVSroot_directory, ".", *pargc <= 1);
-
-	/*
-	 * build dirs on the path if necessary and leave us in the bottom
-	 * directory (where if where was specified) doesn't contain a CVS
-	 * subdir yet, but all the others contain CVS and Entries.Static
-	 * files
-	 */
-	if (build_dirs_and_chdir (head, *pargc <= 1) != 0)
 	{
-	    error (0, 0, "ignoring module %s", omodule);
-	    err = 1;
-	    goto out;
+	    int where_is_absolute = isabsolute (where);
+	    
+	    /* The top-level CVSADM directory should always be
+	       CVSroot_directory.  Create it, but only if WHERE is
+	       relative.  If WHERE is absolute, our current directory
+	       may not have a thing to do with where the sources are
+	       being checked out.  If it does, build_dirs_and_chdir
+	       will take care of creating adm files here. */
+	    /* FIXME: checking where_is_absolute is a horrid kludge;
+	       I suspect we probably can just skip the call to
+	       build_one_dir whenever the -d command option was specified
+	       to checkout.  */
+
+	    if (! where_is_absolute && top_level_admin)
+	    {
+		/* It may be argued that we shouldn't set any sticky
+		   bits for the top-level repository.  FIXME?  */
+		build_one_dir (CVSroot_directory, ".", *pargc <= 1);
+
+#ifdef SERVER_SUPPORT
+		/* We _always_ want to have a top-level admin
+		   directory.  If we're running in client/server mode,
+		   send a "Clear-static-directory" command to make
+		   sure it is created on the client side.  (See 5.10
+		   in cvsclient.dvi to convince yourself that this is
+		   OK.)  If this is a duplicate command being sent, it
+		   will be ignored on the client side.  */
+
+		if (server_active)
+		    server_clear_entstat (".", CVSroot_directory);
+#endif
+	    }
+
+
+	    /* Build dirs on the path if necessary and leave us in the
+	       bottom directory (where if where was specified) doesn't
+	       contain a CVS subdir yet, but all the others contain
+	       CVS and Entries.Static files */
+
+	    if (build_dirs_and_chdir (head, *pargc <= 1) != 0)
+	    {
+		error (0, 0, "ignoring module %s", omodule);
+		err = 1;
+		goto out;
+	    }
 	}
 
 	/* set up the repository (or make sure the old one matches) */
@@ -969,7 +1049,7 @@ internal error: %s doesn't start with %s in checkout_proc",
 	List *entries;
 
 	/* we are only doing files, so register them */
-	entries = Entries_Open (0);
+	entries = Entries_Open (0, NULL);
 	for (i = 1; i < *pargc; i++)
 	{
 	    char *line;
@@ -1081,8 +1161,12 @@ build_dirs_and_chdir (dirs, sticky)
     {
 	char *dir = last_component (dirs->dirpath);
 
-	mkdir_if_needed (dir);
-	Subdir_Register (NULL, NULL, dir);
+	if (!dirs->just_chdir)
+	{
+	    mkdir_if_needed (dir);
+	    Subdir_Register (NULL, NULL, dir);
+	}
+
 	if (CVS_CHDIR (dir) < 0)
 	{
 	    error (0, errno, "cannot chdir to %s", dir);
