@@ -3,7 +3,7 @@
  * Copyright (c) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
- * specified in the README file that comes with the CVS 1.4 kit.
+ * specified in the README file that comes with the CVS source distribution.
  * 
  * Create Administration.
  * 
@@ -13,35 +13,43 @@
 
 #include "cvs.h"
 
-/* update_dir includes dir as its last component.  */
+/* update_dir includes dir as its last component.
 
-void
-Create_Admin (dir, update_dir, repository, tag, date)
+   Return value is 0 for success, or 1 if we printed a warning message.
+   Note that many errors are still fatal; particularly for unlikely errors
+   a fatal error is probably better than a warning which might be missed
+   or after which CVS might do something non-useful.  If WARN is zero, then
+   don't print warnings; all errors are fatal then.  */
+
+int
+Create_Admin (dir, update_dir, repository, tag, date, nonbranch, warn)
     char *dir;
     char *update_dir;
     char *repository;
     char *tag;
     char *date;
+    int nonbranch;
+    int warn;
 {
     FILE *fout;
     char *cp;
-    char tmp[PATH_MAX];
+    char *reposcopy;
+    char *tmp;
 
 #ifdef SERVER_SUPPORT
     if (trace)
     {
-	char wd[PATH_MAX];
-	getwd (wd);
-	fprintf (stderr, "%c-> Create_Admin (%s, %s, %s, %s, %s) in %s\n",
+	fprintf (stderr, "%c-> Create_Admin (%s, %s, %s, %s, %s, %d, %d)\n",
 		 (server_active) ? 'S' : ' ',
-                dir, update_dir, repository, tag ? tag : "",
-                date ? date : "", wd);
+		 dir, update_dir, repository, tag ? tag : "",
+		 date ? date : "", nonbranch, warn);
     }
 #endif
 
     if (noexec)
-	return;
+	return 0;
 
+    tmp = xmalloc (strlen (dir) + 100);
     if (dir != NULL)
 	(void) sprintf (tmp, "%s/%s", dir, CVSADM);
     else
@@ -49,7 +57,30 @@ Create_Admin (dir, update_dir, repository, tag, date)
     if (isfile (tmp))
 	error (1, 0, "there is a version in %s already", update_dir);
 
-    make_directory (tmp);
+    if (CVS_MKDIR (tmp, 0777) < 0)
+    {
+	/* We want to print out the entire update_dir, since a lot of
+	   our code calls this function with dir == "." or dir ==
+	   NULL.  I hope that gives enough information in cases like
+	   absolute pathnames; printing out xgetwd or something would
+	   be way too verbose in the common cases.  */
+
+	if (warn)
+	{
+	    /* The reason that this is a warning, rather than silently
+	       just skipping creating the directory, is that we don't want
+	       CVS's behavior to vary subtly based on factors (like directory
+	       permissions) which are not made clear to the user.  With
+	       the warning at least we let them know what is going on.  */
+	    error (0, errno, "warning: cannot make directory %s in %s",
+		   CVSADM, update_dir);
+	    free (tmp);
+	    return 1;
+	}
+	else
+	    error (1, errno, "cannot make directory %s in %s",
+		   CVSADM, update_dir);
+    }
 
     /* record the current cvs root for later use */
 
@@ -66,8 +97,22 @@ Create_Admin (dir, update_dir, repository, tag, date)
 	else
 	    error (1, errno, "cannot open %s/%s", update_dir, CVSADM_REP);
     }
-    cp = repository;
-    strip_path (cp);
+    reposcopy = xstrdup (repository);
+    Sanitize_Repository_Name (reposcopy);
+
+    /* The top level of the repository is a special case -- we need to
+       write it with an extra dot at the end.  This trailing `.' stuff
+       rubs me the wrong way -- on the other hand, I don't want to
+       spend the time making sure all of the code can handle it if we
+       don't do it. */
+
+    if (strcmp (reposcopy, CVSroot_directory) == 0)
+    {
+	reposcopy = xrealloc (reposcopy, strlen (reposcopy) + 3);
+	strcat (reposcopy, "/.");
+    }
+
+    cp = reposcopy;
 
 #ifdef RELATIVE_REPOS
     /*
@@ -76,11 +121,12 @@ Create_Admin (dir, update_dir, repository, tag, date)
      */
     if (CVSroot_directory != NULL)
     {
-	char path[PATH_MAX];
+	char *path = xmalloc (strlen (CVSroot_directory) + 10);
 
 	(void) sprintf (path, "%s/", CVSroot_directory);
-	if (strncmp (repository, path, strlen (path)) == 0)
-	    cp = repository + strlen (path);
+	if (strncmp (cp, path, strlen (path)) == 0)
+	    cp += strlen (path);
+	free (path);
     }
 #endif
 
@@ -121,12 +167,11 @@ Create_Admin (dir, update_dir, repository, tag, date)
     }
 
     /* Create a new CVS/Tag file */
-    WriteTag (dir, tag, date);
+    WriteTag (dir, tag, date, nonbranch, update_dir, repository);
 
 #ifdef SERVER_SUPPORT
     if (server_active)
     {
-	server_set_sticky (update_dir, repository, tag, date);
 	server_template (update_dir, repository);
     }
 
@@ -137,4 +182,7 @@ Create_Admin (dir, update_dir, repository, tag, date)
     }
 #endif
 
+    free (reposcopy);
+    free (tmp);
+    return 0;
 }
