@@ -2,7 +2,7 @@
  * Copyright (c) 1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
- * specified in the README file that comes with the CVS 1.4 kit.
+ * specified in the README file that comes with the CVS source distribution.
  * 
  * A simple ndbm-emulator for CVS.  It parses a text file of the format:
  * 
@@ -19,8 +19,10 @@
 
 #ifdef MY_NDBM
 
-static void mydbm_load_file ();
+static void mydbm_load_file PROTO ((FILE *, List *));
 
+/* Returns NULL on error in which case errno has been set to indicate
+   the error.  Can also call error() itself.  */
 /* ARGSUSED */
 DBM *
 mydbm_open (file, flags, mode)
@@ -198,52 +200,66 @@ mydbm_load_file (fp, list)
     List *list;
 {
     char *line = NULL;
-    size_t line_len;
-    /* FIXME: arbitrary limit.  */
-    char value[MAXLINELEN];
+    size_t line_size;
+    char *value;
+    size_t value_allocated;
     char *cp, *vp;
-    int len, cont;
+    int cont;
+    int line_length;
 
-    for (cont = 0; getline (&line, &line_len, fp) >= 0;)
+    value_allocated = 1;
+    value = xmalloc (value_allocated);
+
+    cont = 0;
+    while ((line_length = getstr (&line, &line_size, fp, '\012', 0)) >= 0)
     {
-	if ((cp = strrchr (line, '\012')) != NULL)
-	    *cp = '\0';			/* strip the newline */
-	cp = line + strlen (line);
-	if (cp > line && cp[-1] == '\015')
+	if (line_length > 0 && line[line_length - 1] == '\012')
+	{
+	    /* Strip the newline.  */
+	    --line_length;
+	    line[line_length] = '\0';
+	}
+	if (line_length > 0 && line[line_length - 1] == '\015')
+	{
 	    /* If the file (e.g. modules) was written on an NT box, it will
 	       contain CRLF at the ends of lines.  Strip them (we can't do
 	       this by opening the file in text mode because we might be
 	       running on unix).  */
-	    cp[-1] = '\0';	    
+	    --line_length;
+	    line[line_length] = '\0';
+	}
 
 	/*
 	 * Add the line to the value, at the end if this is a continuation
 	 * line; otherwise at the beginning, but only after any trailing
 	 * backslash is removed.
 	 */
-	vp = value;
-	if (cont)
-	    vp += strlen (value);
+	if (!cont)
+	    value[0] = '\0';
 
 	/*
 	 * See if the line we read is a continuation line, and strip the
 	 * backslash if so.
 	 */
-	len = strlen (line);
-	if (len > 0)
-	    cp = &line[len - 1];
+	if (line_length > 0)
+	    cp = &line[line_length - 1];
 	else
 	    cp = line;
 	if (*cp == '\\')
 	{
 	    cont = 1;
 	    *cp = '\0';
+	    --line_length;
 	}
 	else
 	{
 	    cont = 0;
 	}
-	(void) strcpy (vp, line);
+	expand_string (&value,
+		       &value_allocated,
+		       strlen (value) + line_length + 5);
+	strcat (value, line);
+
 	if (value[0] == '#')
 	    continue;			/* comment line */
 	vp = value;
@@ -282,7 +298,12 @@ mydbm_load_file (fp, list)
 	    }
 	}
     }
+    if (line_length < 0 && !feof (fp))
+	/* FIXME: should give the name of the file.  */
+	error (0, errno, "cannot read file in mydbm_load_file");
+
     free (line);
+    free (value);
 }
 
 #endif				/* MY_NDBM */
