@@ -386,6 +386,7 @@ patch_fileproc (callerdat, finfo)
     size_t line2_chars_allocated;
     char *cp1, *cp2;
     FILE *fp;
+    int line_length;
 
     line1 = NULL;
     line1_chars_allocated = 0;
@@ -498,23 +499,43 @@ patch_fileproc (callerdat, finfo)
 	ret = 0;
 	goto out2;
     }
+
+    /* Create 3 empty files.  I'm not really sure there is any advantage
+       to doing so now rather than just waiting until later.  */
     tmpfile1 = cvs_temp_name ();
-    if ((fp1 = CVS_FOPEN (tmpfile1, "w+")) != NULL)
-	(void) fclose (fp1);
-    tmpfile2 = cvs_temp_name ();
-    if ((fp2 = CVS_FOPEN (tmpfile2, "w+")) != NULL)
-	(void) fclose (fp2);
-    tmpfile3 = cvs_temp_name ();
-    if ((fp3 = CVS_FOPEN (tmpfile3, "w+")) != NULL)
-	(void) fclose (fp3);
-    if (fp1 == NULL || fp2 == NULL || fp3 == NULL)
+    fp1 = CVS_FOPEN (tmpfile1, "w+");
+    if (fp1 == NULL)
     {
-	/* FIXME: should be printing a proper error message, with errno-based
-	   message, and the filename which we could not create.  */
-	error (0, 0, "cannot create temporary files");
+	error (0, errno, "cannot create temporary file %s", tmpfile1);
 	ret = 1;
 	goto out;
     }
+    else
+	if (fclose (fp1) < 0)
+	    error (0, errno, "warning: cannot close %s", tmpfile1);
+    tmpfile2 = cvs_temp_name ();
+    fp2 = CVS_FOPEN (tmpfile2, "w+");
+    if (fp2 == NULL)
+    {
+	error (0, errno, "cannot create temporary file %s", tmpfile2);
+	ret = 1;
+	goto out;
+    }
+    else
+	if (fclose (fp2) < 0)
+	    error (0, errno, "warning: cannot close %s", tmpfile2);
+    tmpfile3 = cvs_temp_name ();
+    fp3 = CVS_FOPEN (tmpfile3, "w+");
+    if (fp3 == NULL)
+    {
+	error (0, errno, "cannot create temporary file %s", tmpfile3);
+	ret = 1;
+	goto out;
+    }
+    else
+	if (fclose (fp3) < 0)
+	    error (0, errno, "warning: cannot close %s", tmpfile3);
+
     if (vers_tag != NULL)
     {
 	retcode = RCS_checkout (rcsfile, (char *) NULL, vers_tag,
@@ -522,9 +543,8 @@ patch_fileproc (callerdat, finfo)
 				(RCSCHECKOUTPROC) NULL, (void *) NULL);
 	if (retcode != 0)
 	{
-	    if (!really_quiet)
-		error (retcode == -1 ? 1 : 0, retcode == -1 ? errno : 0,
-		       "co of revision %s in %s failed", vers_tag, rcs);
+	    error (0, 0,
+		   "cannot check out revision %s of %s", vers_tag, rcs);
 	    ret = 1;
 	    goto out;
 	}
@@ -547,9 +567,8 @@ patch_fileproc (callerdat, finfo)
 				(RCSCHECKOUTPROC) NULL, (void *) NULL);
 	if (retcode != 0)
 	{
-	    if (!really_quiet)
-		error (retcode == -1 ? 1 : 0, retcode == -1 ? errno : 0,
-		       "co of revision %s in %s failed", vers_head, rcs);
+	    error (0, 0,
+		   "cannot check out revision %s of %s", vers_head, rcs);
 	    ret = 1;
 	    goto out;
 	}
@@ -583,10 +602,16 @@ patch_fileproc (callerdat, finfo)
 	    if (getline (&line1, &line1_chars_allocated, fp) < 0 ||
 		getline (&line2, &line2_chars_allocated, fp) < 0)
 	    {
-		error (0, errno, "failed to read diff file header %s for %s",
-		       tmpfile3, rcs);
+		if (feof (fp))
+		    error (0, 0, "\
+failed to read diff file header %s for %s: end of file", tmpfile3, rcs);
+		else
+		    error (0, errno,
+			   "failed to read diff file header %s for %s",
+			   tmpfile3, rcs);
 		ret = 1;
-		(void) fclose (fp);
+		if (fclose (fp) < 0)
+		    error (0, errno, "error closing %s", tmpfile3);
 		goto out;
 	    }
 	    if (!unidiff)
@@ -598,7 +623,8 @@ patch_fileproc (callerdat, finfo)
 		{
 		    error (0, 0, "invalid diff header for %s", rcs);
 		    ret = 1;
-		    (void) fclose (fp);
+		    if (fclose (fp) < 0)
+			error (0, errno, "error closing %s", tmpfile3);
 		    goto out;
 		}
 	    }
@@ -611,7 +637,8 @@ patch_fileproc (callerdat, finfo)
 		{
 		    error (0, 0, "invalid unidiff header for %s", rcs);
 		    ret = 1;
-		    (void) fclose (fp);
+		    if (fclose (fp) < 0)
+			error (0, errno, "error closing %s", tmpfile3);
 		    goto out;
 		}
 	    }
@@ -676,10 +703,15 @@ patch_fileproc (callerdat, finfo)
 	    cvs_output (cp2, 0);
 
 	    /* spew the rest of the diff out */
-	    while (getline (&line1, &line1_chars_allocated, fp) >= 0)
+	    while ((line_length
+		    = getline (&line1, &line1_chars_allocated, fp))
+		   >= 0)
 		cvs_output (line1, 0);
+	    if (line_length < 0 && !feof (fp))
+		error (0, errno, "cannot read %s", tmpfile3);
 
-	    (void) fclose (fp);
+	    if (fclose (fp) < 0)
+		error (0, errno, "cannot close %s", tmpfile3);
 	    free (file1);
 	    free (file2);
 	    break;
@@ -691,10 +723,12 @@ patch_fileproc (callerdat, finfo)
         free (line1);
     if (line2)
         free (line2);
-    /* FIXME: should be checking for errors.  */
-    (void) CVS_UNLINK (tmpfile1);
-    (void) CVS_UNLINK (tmpfile2);
-    (void) CVS_UNLINK (tmpfile3);
+    if (CVS_UNLINK (tmpfile1) < 0)
+	error (0, errno, "cannot unlink %s", tmpfile1);
+    if (CVS_UNLINK (tmpfile2) < 0)
+	error (0, errno, "cannot unlink %s", tmpfile2);
+    if (CVS_UNLINK (tmpfile3) < 0)
+	error (0, errno, "cannot unlink %s", tmpfile3);
     free (tmpfile1);
     free (tmpfile2);
     free (tmpfile3);
