@@ -1,7 +1,7 @@
 /* Determined from CC RTL function prototypes in online documentation */
 
 #define mode_t unsigned int
-     
+
 #define fork(x) vfork(x)
 
 #include <sys/types.h>
@@ -17,15 +17,18 @@
 
 extern int fnmatch(char *pattern, char *string, int options);
 
+/* With the define of HAVE_NDIR_H in config.h, lib/system.h and
+   diff/system.h should include ndir.h for us.  But I'm too lazy to
+   track down and make _sure_ all bases are covered, so I'm leaving in
+   this include for now.  */
 #include "ndir.h"
+
 #include "pwd.h"
 #include "pipe.h"
 
 int unlink(char *path);
 int link(char *from, char *to);
 
-int rmdir(char *path);
-
 #define stat(a, b) wrapped_stat(a, b)
 
 #undef POSIX
